@@ -135,13 +135,19 @@ has value =>
 	required => 0,
 );
 
-our $VERSION = '1.03';
+# This flag is used to stop Perl issuing deep recursion warnings,
+# when Tree::DAG_Node's walk_down() traverses the parse tree.
+
+my($sig_warn_flag) = 1;
+
+our $VERSION = '1.04';
 
 # ------------------------------------------------
 
 sub BUILD
 {
-	my($self) = @_;
+	my($self)        = @_;
+	$SIG{'__WARN__'} = sub { warn $_[0] if $sig_warn_flag};
 
 	if (! defined $self -> logger)
 	{
@@ -167,7 +173,8 @@ sub BUILD
 	(
 		Marpa::R2::Scanless::R -> new
 		({
-			grammar => $self -> grammar,
+			grammar        => $self -> grammar,
+			ranking_method => 'high_rule_only',
 		})
 	);
 	$self -> renderer
@@ -275,7 +282,7 @@ sub process
 				if ( exists $self -> keywords -> { $literal } )
 				{
 					$self -> recce -> lexeme_read(qq{keyword $literal}, $start, $length)
-					// die $self->input_file_name . qq{:$line:$column: keyword '$literal' used as <name>\n};
+					// die $self->input_file_name . qq{ (line, column) = ($line, $column): keyword '$literal' used as <name>\n};
                 }
                 else
                 {
@@ -314,6 +321,7 @@ sub render
 {
 	my($self)      = @_;
 	my($slim_list) = [];
+	$sig_warn_flag = 0;
 
 	my($attributes);
 	my($name);
@@ -358,12 +366,15 @@ sub run
 {
 	my($self, %args) = @_;
 	my($file_name)   = $args{input_file_name} || $self -> input_file_name;
+	$sig_warn_flag   = 0; # Turn off Perl's warnings for the duration of tree processing.
 
 	$self -> value($self -> process($file_name) );
 	$self -> renderer -> run(${$self -> value});
 	$self -> log(debug => $_) for @{$self -> renderer -> root -> tree2string({no_attributes => 1 - $self -> attributes})};
 	$self -> render;
 
+	$sig_warn_flag = 1; # Turn Perl's warnings back on.
+
 	my($output_file_name) = $args{output_file_name} || $self -> output_file_name;
 
 	if ($output_file_name)
@@ -742,25 +753,29 @@ __DATA__
 
 lexeme default = latm => 1 action => [name,values]
 
-# I(JK)  attempt to follow the order of the Lua grammar in
+# I (JK)  attempt to follow the order of the Lua grammar in
 # section 8 of the Lua 5.1 reference manual.
 #
 # Names which begin with "Lua" are taken directly from
 # the Lua reference manual grammar.
 
-<chunk> ::= <stat list> <optional laststat>
-<stat list> ::= <stat item>*
-<stat item> ::= <stat> ';'
-<stat item> ::= <stat>
-<optional laststat> ::= <laststat> ';'
-<optional laststat> ::= <laststat>
-<optional laststat> ::=
+<chunk> ::=
+<chunk> ::= <stat list>
+<chunk> ::= <stat list> laststat
+<chunk> ::= <stat list> laststat ';'
+<chunk> ::= laststat ';'
+<chunk> ::= laststat
+
+<stat list> ::= <stat>
+<stat list> ::= <stat> ';'
+<stat list> ::= <stat list> <stat> rank => -1
+<stat list> ::= <stat list> <stat> ';'
 
 <block> ::= <chunk>
 
 <stat> ::= <varlist> '=' <explist>
 
-<stat> ::= <functioncall>
+<stat> ::= <functioncall> rank => -1
 
 <stat> ::= <keyword do> <block> <keyword end>
 
@@ -770,14 +785,17 @@ lexeme default = latm => 1 action => [name,values]
 
 <stat> ::= <keyword if> <exp> <keyword then> <block>
     <elseif sequence> <optional else block> <keyword end>
+
 <elseif sequence> ::= <elseif sequence> <elseif block>
 <elseif sequence> ::=
 <elseif block> ::= <keyword elseif> <exp> <keyword then> <block>
+
 <optional else block> ::= <keyword else> <block>
 <optional else block> ::=
 
 <stat> ::= <keyword for> <Name> '=' <exp> ',' <exp> ',' <exp>
     <keyword do> <block> <keyword end>
+
 <stat> ::= <keyword for> <Name> '=' <exp> ',' <exp> <keyword do> <block> <keyword end>
 
 <stat> ::= <keyword for> <namelist> <keyword in> <explist> <keyword do> <block> <keyword end>
@@ -798,7 +816,9 @@ lexeme default = latm => 1 action => [name,values]
 <optional explist> ::= <explist>
 
 <funcname> ::= <dotted name> <optional colon name element>
+
 <dotted name> ::= <Name>+ separator => [.] proper => 1
+
 <optional colon name element> ::=
 <optional colon name element> ::= ':' <Name>
 
@@ -813,36 +833,53 @@ lexeme default = latm => 1 action => [name,values]
 <explist> ::= <exp>+ separator => [,] proper => 1
 
 <exp> ::=
-       <var>
-     | '(' <exp> ')' assoc => group
-    || <exp> <args> assoc => right
-    || <exp> ':' <Name> <args> assoc => right
-     | <keyword nil>
-     | <keyword false>
-     | <keyword true>
-     | <Number>
-     | <String>
-     | '...'
-     | <tableconstructor>
-     | <function>
-    || <exp> '^' <exp> assoc => right
-    || <keyword not> <exp>
-     | '#' <exp>
-     | '-' <exp>
-    || <exp> '*' <exp>
-     | <exp> '/' <exp>
-     | <exp> '%' <exp>
-    || <exp> '+' <exp>
-     | <exp> '-' <exp>
-    || <exp> '..' <exp> assoc => right
-    || <exp> '<' <exp>
-     | <exp> '<=' <exp>
-     | <exp> '>' <exp>
-     | <exp> '>=' <exp>
-     | <exp> '==' <exp>
-     | <exp> '~=' <exp>
-    || <exp> <keyword and> <exp>
-    || <exp> <keyword or> <exp>
+		<var>
+	 | '(' <exp> ')' assoc => group
+	|| <exp> <args> assoc => right
+	|| <exp> ':' <Name> <args> assoc => right
+	 | <keyword nil>
+	 | <keyword false>
+	 | <keyword true>
+	 | <Number>
+	 | <String>
+	 | '...'
+	 | <tableconstructor>
+	 | <function>
+	|| <exp> '^' <exponent> assoc => right
+	|| '-' <exp>
+	 | <keyword not> <exp>
+	 | '#' <exp>
+	|| <exp> '*' <exp>
+	 | <exp> '/' <exp>
+	 | <exp> '%' <exp>
+	|| <exp> '+' <exp>
+	 | <exp> '-' <exp>
+	|| <exp> '..' <exp> assoc => right
+	|| <exp> '<' <exp>
+	 | <exp> '<=' <exp>
+	 | <exp> '>' <exp>
+	 | <exp> '>=' <exp>
+	 | <exp> '==' <exp> rank => 1
+	 | <exp> '~=' <exp>
+	|| <exp> <keyword and> <exp> rank => 1
+	|| <exp> <keyword or> <exp>
+
+<exponent> ::=
+		<var>
+	 | '(' <exp> ')'
+	|| <exponent> <args>
+	|| <exponent> ':' <Name> <args>
+	 | <keyword nil>
+	 | <keyword false>
+	 | <keyword true>
+	 | <Number>
+	 | <String>
+	 | '...'
+	 | <tableconstructor>
+	 | <function>
+	|| <keyword not> <exponent>
+	 | '#' <exponent>
+	 | '-' <exponent>
 
 <prefixexp> ::= <var>
 <prefixexp> ::= <functioncall>
@@ -871,6 +908,7 @@ lexeme default = latm => 1 action => [name,values]
 
 <tableconstructor> ::= '{' '}'
 <tableconstructor> ::= '{' <fieldlist> '}'
+
 <fieldlist> ::= <field>+ separator => [,;]
 
 <field> ::= '[' <exp> ']' '=' <exp>
@@ -904,10 +942,12 @@ lexeme default = latm => 1 action => [name,values]
 # of the comment
 
 :discard ~ <singleline comment> event => 'singleline comment'
+
 <singleline comment> ~ <singleline comment start>
 <singleline comment start> ~ '--'
 
 :discard ~ <multiline comment> event => 'multiline comment'
+
 <multiline comment> ~ '--[' <optional equal signs> '['
 
 <optional equal signs> ~ [=]*
@@ -924,13 +964,19 @@ whitespace ~ [\s]+
 # and we enforce that, so all letters must be a-z or A-Z
 
 <Name> ~ <identifier start char> <optional identifier chars>
+
 :lexeme ~ Name pause => before event => 'Name'
+
 <identifier start char> ~ [a-zA-Z_]
+
 <optional identifier chars> ~ <identifier char>*
+
 <identifier char> ~ [a-zA-Z0-9_]
 
 <String> ::= <single quoted string>
+
 <single quoted string> ~ ['] <optional single quoted chars> [']
+
 <optional single quoted chars> ~ <single quoted char>*
 
 # anything other than vertical space or a single quote
@@ -939,7 +985,9 @@ whitespace ~ [\s]+
 <single quoted char> ~ '\' [\d\D] # Also an escaped char. Another ' for uex.
 
 <String> ::= <double quoted string>
+
 <double quoted string> ~ ["] <optional double quoted chars> ["]
+
 <optional double quoted chars> ~ <double quoted char>*
 
 # anything other than vertical space or a double quote
@@ -948,7 +996,9 @@ whitespace ~ [\s]+
 <double quoted char> ~ '\' [\d\D] # also an escaped char. Another ' for uex.
 
 <String> ::= <multiline string>
+
 :lexeme ~ <multiline string> pause => before event => 'multiline string'
+
 <multiline string> ~ '[' <optional equal signs> '['
 
 <Number> ~ <hex number>
@@ -969,10 +1019,14 @@ whitespace ~ [\s]+
 <C90 strtod decimal> ~ <optional sign> '.' <decimal digits> <optional exponent>
 <C90 strtod decimal> ~
     <optional sign> <decimal digits> '.' <decimal digits> <optional exponent>
+
 <optional exponent> ~
 <optional exponent> ~ [eE] <optional sign> <decimal digits>
 <optional sign> ~
 <optional sign> ~ [-+]
+
 <C90 strtol hex> ~ [0] [xX] <hex digits>
+
 <decimal digits> ~ [0-9]+
+
 <hex digits> ~ [a-fA-F0-9]+
